@@ -30,8 +30,10 @@
 
 #endif
 
+#include "util.hpp"
 #include "units.hpp"
-#include "token.hpp"
+#include "context.hpp"
+#include "position.hpp"
 #include "constants.hpp"
 #include "operation.hpp"
 #include "position.hpp"
@@ -40,7 +42,9 @@
 #include "environment.hpp"
 #include "error_handling.hpp"
 #include "ast_def_macros.hpp"
- #include "to_string.hpp"
+#include "ast_fwd_decl.hpp"
+#include "to_string.hpp"
+#include "source_map.hpp"
 
 #include "sass.h"
 #include "sass_values.h"
@@ -52,14 +56,12 @@ namespace Sass {
   //////////////////////////////////////////////////////////
   // Abstract base class for all abstract syntax tree nodes.
   //////////////////////////////////////////////////////////
-  class Block;
-  class Statement;
-  class Expression;
-  class Selector;
   class AST_Node {
     ADD_PROPERTY(ParserState, pstate);
   public:
-    AST_Node(ParserState pstate) : pstate_(pstate) { }
+    AST_Node(ParserState pstate)
+    : pstate_(pstate)
+    { }
     virtual ~AST_Node() = 0;
     // virtual Block* block() { return 0; }
   public:
@@ -85,6 +87,7 @@ namespace Sass {
       STRING,
       LIST,
       MAP,
+      SELECTOR,
       NULL_VAL,
       NUM_TYPES
     };
@@ -98,7 +101,10 @@ namespace Sass {
     Expression(ParserState pstate,
                bool d = false, bool e = false, bool i = false, Concrete_Type ct = NONE)
     : AST_Node(pstate),
-      is_delayed_(d), is_expanded_(d), is_interpolant_(i), concrete_type_(ct)
+      is_delayed_(d),
+      is_expanded_(d),
+      is_interpolant_(i),
+      concrete_type_(ct)
     { }
     virtual operator bool() { return true; }
     virtual ~Expression() { };
@@ -184,7 +190,7 @@ namespace Sass {
   inline Vectorized<T>::~Vectorized() { }
 
   /////////////////////////////////////////////////////////////////////////////
-  // Mixin class for AST nodes that should behave like ahash table. Uses an
+  // Mixin class for AST nodes that should behave like a hash table. Uses an
   // extra <vector> internally to maintain insertion order for interation.
   /////////////////////////////////////////////////////////////////////////////
   class Hashed {
@@ -204,9 +210,10 @@ namespace Sass {
     size_t length() const                  { return list_.size(); }
     bool empty() const                     { return list_.empty(); }
     bool has(Expression* k) const          { return elements_.count(k) == 1; }
-    Expression* at(Expression* k) const    { return elements_.at(k); }
+    Expression* at(Expression* k) const;
     bool has_duplicate_key() const         { return duplicate_key_ != 0; }
     Expression* get_duplicate_key() const  { return duplicate_key_; }
+    const unordered_map<Expression*, Expression*> elements() { return elements_; }
     Hashed& operator<<(pair<Expression*, Expression*> p)
     {
       reset_hash();
@@ -316,7 +323,6 @@ namespace Sass {
   // Rulesets (i.e., sets of styles headed by a selector and containing a block
   // of style declarations.
   /////////////////////////////////////////////////////////////////////////////
-  class Selector;
   class Ruleset : public Has_Block {
     ADD_PROPERTY(Selector*, selector);
   public:
@@ -332,7 +338,6 @@ namespace Sass {
   /////////////////////////////////////////////////////////
   // Nested declaration sets (i.e., namespaced properties).
   /////////////////////////////////////////////////////////
-  class String;
   class Propset : public Has_Block {
     ADD_PROPERTY(String*, property_fragment);
   public:
@@ -350,7 +355,7 @@ namespace Sass {
     ADD_PROPERTY(bool, group_end);
   public:
     Bubble(ParserState pstate, Statement* n, Statement* g = 0, size_t t = 0)
-    : Statement(pstate, Statement::BUBBLE, t), node_(n), group_end_(g)
+    : Statement(pstate, Statement::BUBBLE, t), node_(n), group_end_(g == 0)
     { }
     bool bubbles() { return true; }
     ATTACH_OPERATIONS();
@@ -359,7 +364,6 @@ namespace Sass {
   /////////////////
   // Media queries.
   /////////////////
-  class List;
   class Media_Block : public Has_Block {
     ADD_PROPERTY(List*, media_queries);
     ADD_PROPERTY(Selector*, selector);
@@ -408,7 +412,13 @@ namespace Sass {
     At_Rule(ParserState pstate, string kwd, Selector* sel = 0, Block* b = 0)
     : Has_Block(pstate, b), keyword_(kwd), selector_(sel), value_(0) // set value manually if needed
     { statement_type(DIRECTIVE); }
-    bool bubbles() { return true; }
+    bool bubbles() { return is_keyframes() || is_media(); }
+    bool is_media() {
+      return keyword_.compare("@-webkit-media") == 0 ||
+             keyword_.compare("@-moz-media") == 0 ||
+             keyword_.compare("@-o-media") == 0 ||
+             keyword_.compare("@media") == 0;
+    }
     bool is_keyframes() {
       return keyword_.compare("@-webkit-keyframes") == 0 ||
              keyword_.compare("@-moz-keyframes") == 0 ||
@@ -422,10 +432,10 @@ namespace Sass {
   // Keyframe-rules -- the child blocks of "@keyframes" nodes.
   ///////////////////////////////////////////////////////////////////////
   class Keyframe_Rule : public Has_Block {
-    ADD_PROPERTY(String*, rules);
+    ADD_PROPERTY(Selector*, selector);
   public:
     Keyframe_Rule(ParserState pstate, Block* b)
-    : Has_Block(pstate, b), rules_(0)
+    : Has_Block(pstate, b), selector_(0)
     { statement_type(KEYFRAMERULE); }
     ATTACH_OPERATIONS();
   };
@@ -448,19 +458,17 @@ namespace Sass {
   /////////////////////////////////////
   // Assignments -- variable and value.
   /////////////////////////////////////
-  class Variable;
-  class Expression;
   class Assignment : public Statement {
     ADD_PROPERTY(string, variable);
     ADD_PROPERTY(Expression*, value);
-    ADD_PROPERTY(bool, is_guarded);
+    ADD_PROPERTY(bool, is_default);
     ADD_PROPERTY(bool, is_global);
   public:
     Assignment(ParserState pstate,
                string var, Expression* val,
-               bool guarded = false,
-               bool global = false)
-    : Statement(pstate), variable_(var), value_(val), is_guarded_(guarded), is_global_(global)
+               bool is_default = false,
+               bool is_global = false)
+    : Statement(pstate), variable_(var), value_(val), is_default_(is_default), is_global_(is_global)
     { }
     ATTACH_OPERATIONS();
   };
@@ -471,7 +479,7 @@ namespace Sass {
   ////////////////////////////////////////////////////////////////////////////
   class Import : public Statement {
     vector<string>         files_;
-    vector<Expression*> urls_;
+    vector<Expression*>    urls_;
   public:
     Import(ParserState pstate)
     : Statement(pstate),
@@ -532,9 +540,10 @@ namespace Sass {
   ///////////////////////////////////////////
   class Comment : public Statement {
     ADD_PROPERTY(String*, text);
+    ADD_PROPERTY(bool, is_important);
   public:
-    Comment(ParserState pstate, String* txt)
-    : Statement(pstate), text_(txt)
+    Comment(ParserState pstate, String* txt, bool is_important)
+    : Statement(pstate), text_(txt), is_important_(is_important)
     { }
     ATTACH_OPERATIONS();
   };
@@ -623,9 +632,7 @@ namespace Sass {
   // Definitions for both mixins and functions. The two cases are distinguished
   // by a type tag.
   /////////////////////////////////////////////////////////////////////////////
-  struct Context;
   struct Backtrace;
-  class Parameters;
   typedef Environment<AST_Node*> Env;
   typedef const char* Signature;
   typedef Expression* (*Native_Function)(Env&, Env&, Context&, Signature, ParserState, Backtrace*);
@@ -638,8 +645,9 @@ namespace Sass {
     ADD_PROPERTY(Env*, environment);
     ADD_PROPERTY(Type, type);
     ADD_PROPERTY(Native_Function, native_function);
-    ADD_PROPERTY(Sass_C_Function, c_function);
+    ADD_PROPERTY(Sass_Function_Entry, c_function);
     ADD_PROPERTY(void*, cookie);
+    ADD_PROPERTY(Context*, ctx);
     ADD_PROPERTY(bool, is_overload_stub);
     ADD_PROPERTY(Signature, signature);
   public:
@@ -647,6 +655,7 @@ namespace Sass {
                string n,
                Parameters* params,
                Block* b,
+               Context* ctx,
                Type t)
     : Has_Block(pstate, b),
       name_(n),
@@ -656,6 +665,7 @@ namespace Sass {
       native_function_(0),
       c_function_(0),
       cookie_(0),
+      ctx_(ctx),
       is_overload_stub_(false),
       signature_(0)
     { }
@@ -664,6 +674,7 @@ namespace Sass {
                string n,
                Parameters* params,
                Native_Function func_ptr,
+               Context* ctx,
                bool overload_stub = false)
     : Has_Block(pstate, 0),
       name_(n),
@@ -673,6 +684,7 @@ namespace Sass {
       native_function_(func_ptr),
       c_function_(0),
       cookie_(0),
+      ctx_(ctx),
       is_overload_stub_(overload_stub),
       signature_(sig)
     { }
@@ -680,8 +692,8 @@ namespace Sass {
                Signature sig,
                string n,
                Parameters* params,
-               Sass_C_Function func_ptr,
-               void* cookie,
+               Sass_Function_Entry c_func,
+               Context* ctx,
                bool whatever,
                bool whatever2)
     : Has_Block(pstate, 0),
@@ -690,8 +702,9 @@ namespace Sass {
       environment_(0),
       type_(FUNCTION),
       native_function_(0),
-      c_function_(func_ptr),
-      cookie_(cookie),
+      c_function_(c_func),
+      cookie_(sass_function_get_cookie(c_func)),
+      ctx_(ctx),
       is_overload_stub_(false),
       signature_(sig)
     { }
@@ -701,7 +714,6 @@ namespace Sass {
   //////////////////////////////////////
   // Mixin calls (i.e., `@include ...`).
   //////////////////////////////////////
-  class Arguments;
   class Mixin_Call : public Has_Block {
     ADD_PROPERTY(string, name);
     ADD_PROPERTY(Arguments*, arguments);
@@ -780,7 +792,6 @@ namespace Sass {
   ///////////////////////////////////////////////////////////////////////
   // Key value paris.
   ///////////////////////////////////////////////////////////////////////
-
   class Map : public Expression, public Hashed {
     void adjust_after_pushing(std::pair<Expression*, Expression*> p) { is_expanded(false); }
   public:
@@ -823,8 +834,6 @@ namespace Sass {
     ATTACH_OPERATIONS();
   };
 
-
-
   //////////////////////////////////////////////////////////////////////////
   // Binary expressions. Represents logical, relational, and arithmetic
   // operations. Templatized to avoid large switch statements and repetitive
@@ -842,11 +851,34 @@ namespace Sass {
     ADD_PROPERTY(Type, type);
     ADD_PROPERTY(Expression*, left);
     ADD_PROPERTY(Expression*, right);
+    size_t hash_;
   public:
     Binary_Expression(ParserState pstate,
                       Type t, Expression* lhs, Expression* rhs)
-    : Expression(pstate), type_(t), left_(lhs), right_(rhs)
+    : Expression(pstate), type_(t), left_(lhs), right_(rhs), hash_(0)
     { }
+    virtual bool operator==(Expression& rhs) const
+    {
+      try
+      {
+        Binary_Expression& m = dynamic_cast<Binary_Expression&>(rhs);
+        if (m == 0) return false;
+        return type() == m.type() &&
+               left() == m.left() &&
+               right() == m.right();
+      }
+      catch (std::bad_cast&)
+      {
+        return false;
+      }
+      catch (...) { throw; }
+    }
+    virtual size_t hash()
+    {
+      if (hash_ > 0) return hash_;
+      hash_ = left()->hash() ^ right()->hash() ^ std::hash<size_t>()(type_);
+      return hash_;
+    }
     ATTACH_OPERATIONS();
   };
 
@@ -859,10 +891,32 @@ namespace Sass {
   private:
     ADD_PROPERTY(Type, type);
     ADD_PROPERTY(Expression*, operand);
+    size_t hash_;
   public:
     Unary_Expression(ParserState pstate, Type t, Expression* o)
-    : Expression(pstate), type_(t), operand_(o)
+    : Expression(pstate), type_(t), operand_(o), hash_(0)
     { }
+    virtual bool operator==(Expression& rhs) const
+    {
+      try
+      {
+        Unary_Expression& m = dynamic_cast<Unary_Expression&>(rhs);
+        if (m == 0) return false;
+        return type() == m.type() &&
+               operand() == m.operand();
+      }
+      catch (std::bad_cast&)
+      {
+        return false;
+      }
+      catch (...) { throw; }
+    }
+    virtual size_t hash()
+    {
+      if (hash_ > 0) return hash_;
+      hash_ = operand()->hash() ^ std::hash<size_t>()(type_);
+      return hash_;
+    }
     ATTACH_OPERATIONS();
   };
 
@@ -890,7 +944,7 @@ namespace Sass {
       {
         Argument& m = dynamic_cast<Argument&>(rhs);
         if (!(m && name() == m.name())) return false;
-        return *value() == *value();
+        return *value() == *m.value();
       }
       catch (std::bad_cast&)
       {
@@ -1326,10 +1380,10 @@ namespace Sass {
   // "flat" strings.
   ////////////////////////////////////////////////////////////////////////
   class String : public Expression {
-    ADD_PROPERTY(bool, needs_unquoting);
+    ADD_PROPERTY(bool, sass_fix_1291);
   public:
-    String(ParserState pstate, bool unq = false, bool delayed = false)
-    : Expression(pstate, delayed), needs_unquoting_(unq)
+    String(ParserState pstate, bool delayed = false, bool sass_fix_1291 = false)
+    : Expression(pstate, delayed), sass_fix_1291_(sass_fix_1291)
     { concrete_type(STRING); }
     static string type_name() { return "string"; }
     virtual ~String() = 0;
@@ -1342,12 +1396,11 @@ namespace Sass {
   // evaluation phase.
   ///////////////////////////////////////////////////////////////////////
   class String_Schema : public String, public Vectorized<Expression*> {
-    ADD_PROPERTY(char, quote_mark);
     ADD_PROPERTY(bool, has_interpolants);
     size_t hash_;
   public:
-    String_Schema(ParserState pstate, size_t size = 0, bool unq = false, char qm = '\0', bool i = false)
-    : String(pstate, unq), Vectorized<Expression*>(size), quote_mark_(qm), has_interpolants_(i), hash_(0)
+    String_Schema(ParserState pstate, size_t size = 0, bool has_interpolants = false)
+    : String(pstate), Vectorized<Expression*>(size), has_interpolants_(has_interpolants), hash_(0)
     { }
     string type() { return "string"; }
     static string type_name() { return "string"; }
@@ -1386,22 +1439,23 @@ namespace Sass {
   // Flat strings -- the lowest level of raw textual data.
   ////////////////////////////////////////////////////////
   class String_Constant : public String {
+    ADD_PROPERTY(char, quote_mark);
     ADD_PROPERTY(string, value);
-    string unquoted_;
+  protected:
     size_t hash_;
   public:
-    String_Constant(ParserState pstate, string val, bool unq = false)
-    : String(pstate, unq, true), value_(val), hash_(0)
-    { unquoted_ = unquote(value_); }
-    String_Constant(ParserState pstate, const char* beg, bool unq = false)
-    : String(pstate, unq, true), value_(string(beg)), hash_(0)
-    { unquoted_ = unquote(value_); }
-    String_Constant(ParserState pstate, const char* beg, const char* end, bool unq = false)
-    : String(pstate, unq, true), value_(string(beg, end-beg)), hash_(0)
-    { unquoted_ = unquote(value_); }
-    String_Constant(ParserState pstate, const Token& tok, bool unq = false)
-    : String(pstate, unq, true), value_(string(tok.begin, tok.end)), hash_(0)
-    { unquoted_ = unquote(value_); }
+    String_Constant(ParserState pstate, string val)
+    : String(pstate), quote_mark_(0), value_(read_css_string(val)), hash_(0)
+    { }
+    String_Constant(ParserState pstate, const char* beg)
+    : String(pstate), quote_mark_(0), value_(read_css_string(string(beg))), hash_(0)
+    { }
+    String_Constant(ParserState pstate, const char* beg, const char* end)
+    : String(pstate), quote_mark_(0), value_(read_css_string(string(beg, end-beg))), hash_(0)
+    { }
+    String_Constant(ParserState pstate, const Token& tok)
+    : String(pstate), quote_mark_(0), value_(read_css_string(string(tok.begin, tok.end))), hash_(0)
+    { }
     string type() { return "string"; }
     static string type_name() { return "string"; }
 
@@ -1410,7 +1464,7 @@ namespace Sass {
       try
       {
         String_Constant& e = dynamic_cast<String_Constant&>(rhs);
-        return e && unquoted_ == e.unquoted_;
+        return e && value_ == e.value_;
       }
       catch (std::bad_cast&)
       {
@@ -1421,15 +1475,27 @@ namespace Sass {
 
     virtual size_t hash()
     {
-      if (hash_ == 0) hash_ = std::hash<string>()(unquoted_);
+      if (hash_ == 0) hash_ = std::hash<string>()(value_);
       return hash_;
     }
 
-    static char single_quote() { return '\''; }
+    // static char auto_quote() { return '*'; }
     static char double_quote() { return '"'; }
+    static char single_quote() { return '\''; }
 
-    bool is_quoted() { return value_.length() && (value_[0] == '"' || value_[0] == '\''); }
-    char quote_mark() { return is_quoted() ? value_[0] : '\0'; }
+    ATTACH_OPERATIONS();
+  };
+
+  ////////////////////////////////////////////////////////
+  // Possibly quoted string (unquote on instantiation)
+  ////////////////////////////////////////////////////////
+  class String_Quoted : public String_Constant {
+  public:
+    String_Quoted(ParserState pstate, string val)
+    : String_Constant(pstate, val)
+    {
+      value_ = unquote(value_, &quote_mark_);
+    }
     ATTACH_OPERATIONS();
   };
 
@@ -1517,7 +1583,7 @@ namespace Sass {
 
       if (with)
       {
-        if (!l || !l->length()) return str.compare("rule");
+        if (!l || l->length() == 0) return str.compare("rule") != 0;
         for (size_t i = 0, L = l->length(); i < L; ++i)
         {
           v = unquote((*l)[i]->perform(&to_string));
@@ -1653,9 +1719,6 @@ namespace Sass {
         if (has_rest_parameter_) {
           error("functions and mixins cannot have more than one variable-length parameter", p->pstate());
         }
-        if (has_optional_parameters_) {
-          error("optional parameters may not be combined with variable-length parameters", p->pstate());
-        }
         has_rest_parameter_ = true;
       }
       else {
@@ -1682,19 +1745,52 @@ namespace Sass {
   //////////////////////////////////////////////////////////////////////////////////////////
   inline Expression* List::value_at_index(size_t i) { return is_arglist_ ? ((Argument*)(*this)[i])->value() : (*this)[i]; }
 
+  ////////////
+  // The Parent Selector Expression.
+  ////////////
+  class Parent_Selector : public Expression {
+    ADD_PROPERTY(Selector*, selector);
+  public:
+    Parent_Selector(ParserState pstate, Selector* r = 0)
+    : Expression(pstate), selector_(r)
+    { concrete_type(SELECTOR); }
+    virtual Selector* selector() { return selector_; }
+    string type() { return "selector"; }
+    static string type_name() { return "selector"; }
+
+    ATTACH_OPERATIONS();
+  };
+
   /////////////////////////////////////////
   // Abstract base class for CSS selectors.
   /////////////////////////////////////////
   class Selector : public AST_Node {
     ADD_PROPERTY(bool, has_reference);
     ADD_PROPERTY(bool, has_placeholder);
+    // line break before list separator
+    ADD_PROPERTY(bool, has_line_feed);
+    // line break after list separator
+    ADD_PROPERTY(bool, has_line_break);
+    // maybe we have optional flag
+    ADD_PROPERTY(bool, is_optional);
+    // parent block pointers
+    ADD_PROPERTY(Block*, last_block);
+    ADD_PROPERTY(Media_Block*, media_block);
   public:
     Selector(ParserState pstate, bool r = false, bool h = false)
-    : AST_Node(pstate), has_reference_(r), has_placeholder_(h)
+    : AST_Node(pstate),
+      has_reference_(r),
+      has_placeholder_(h),
+      has_line_feed_(false),
+      has_line_break_(false),
+      is_optional_(false),
+      media_block_(0)
     { }
     virtual ~Selector() = 0;
-    virtual Selector_Placeholder* find_placeholder();
-    virtual int specificity() { return Constants::SPECIFICITY_BASE; }
+    // virtual Selector_Placeholder* find_placeholder();
+    virtual unsigned long specificity() {
+      return Constants::Specificity_Universal;
+    };
   };
   inline Selector::~Selector() { }
 
@@ -1722,6 +1818,7 @@ namespace Sass {
     virtual ~Simple_Selector() = 0;
     virtual Compound_Selector* unify_with(Compound_Selector*, Context&);
     virtual bool is_pseudo_element() { return false; }
+    virtual bool is_pseudo_class() { return false; }
 
     bool operator==(const Simple_Selector& rhs) const;
     inline bool operator!=(const Simple_Selector& rhs) const { return !(*this == rhs); }
@@ -1739,10 +1836,10 @@ namespace Sass {
     Selector_Reference(ParserState pstate, Selector* r = 0)
     : Simple_Selector(pstate), selector_(r)
     { has_reference(true); }
-    virtual int specificity()
+    virtual unsigned long specificity()
     {
-      if (selector()) return selector()->specificity();
-      else            return 0;
+      if (!selector()) return 0;
+      return selector()->specificity();
     }
     ATTACH_OPERATIONS();
   };
@@ -1756,7 +1853,7 @@ namespace Sass {
     Selector_Placeholder(ParserState pstate, string n)
     : Simple_Selector(pstate), name_(n)
     { has_placeholder(true); }
-    virtual Selector_Placeholder* find_placeholder();
+    // virtual Selector_Placeholder* find_placeholder();
     ATTACH_OPERATIONS();
   };
 
@@ -1769,10 +1866,11 @@ namespace Sass {
     Type_Selector(ParserState pstate, string n)
     : Simple_Selector(pstate), name_(n)
     { }
-    virtual int specificity()
+    virtual unsigned long specificity()
     {
-      if (name() == "*") return 0;
-      else               return 1;
+      // ToDo: What is the specificity of the star selector?
+      if (name() == "*") return Constants::Specificity_Universal;
+      else               return Constants::Specificity_Type;
     }
     virtual Compound_Selector* unify_with(Compound_Selector*, Context&);
     ATTACH_OPERATIONS();
@@ -1787,10 +1885,11 @@ namespace Sass {
     Selector_Qualifier(ParserState pstate, string n)
     : Simple_Selector(pstate), name_(n)
     { }
-    virtual int specificity()
+    virtual unsigned long specificity()
     {
-      if (name()[0] == '#') return Constants::SPECIFICITY_BASE * Constants::SPECIFICITY_BASE;
-      else                  return Constants::SPECIFICITY_BASE;
+      if (name()[0] == '#') return Constants::Specificity_ID;
+      if (name()[0] == '.') return Constants::Specificity_Class;
+      else                  return Constants::Specificity_Type;
     }
     virtual Compound_Selector* unify_with(Compound_Selector*, Context&);
     ATTACH_OPERATIONS();
@@ -1807,12 +1906,28 @@ namespace Sass {
     Attribute_Selector(ParserState pstate, string n, string m, String* v)
     : Simple_Selector(pstate), name_(n), matcher_(m), value_(v)
     { }
+    virtual unsigned long specificity()
+    {
+      return Constants::Specificity_Attr;
+    }
     ATTACH_OPERATIONS();
   };
 
   //////////////////////////////////////////////////////////////////
   // Pseudo selectors -- e.g., :first-child, :nth-of-type(...), etc.
   //////////////////////////////////////////////////////////////////
+  /* '::' starts a pseudo-element, ':' a pseudo-class */
+  /* Except :first-line, :first-letter, :before and :after */
+  /* Note that pseudo-elements are restricted to one per selector */
+  /* and occur only in the last simple_selector_sequence. */
+  inline bool is_pseudo_class_element(const string& name)
+  {
+    return name == ":before"       ||
+           name == ":after"        ||
+           name == ":first-line"   ||
+           name == ":first-letter";
+  }
+
   class Pseudo_Selector : public Simple_Selector {
     ADD_PROPERTY(string, name);
     ADD_PROPERTY(String*, expression);
@@ -1820,29 +1935,33 @@ namespace Sass {
     Pseudo_Selector(ParserState pstate, string n, String* expr = 0)
     : Simple_Selector(pstate), name_(n), expression_(expr)
     { }
-    virtual int specificity()
+
+    // A pseudo-class always consists of a "colon" (:) followed by the name
+    // of the pseudo-class and optionally by a value between parentheses.
+    virtual bool is_pseudo_class()
     {
-      // TODO: clean up the pseudo-element checking
-      if (name() == ":before"       || name() == "::before"     ||
-          name() == ":after"        || name() == "::after"      ||
-          name() == ":first-line"   || name() == "::first-line" ||
-          name() == ":first-letter" || name() == "::first-letter")
-        return 1;
-      else
-        return Constants::SPECIFICITY_BASE;
+      return (name_[0] == ':' && name_[1] != ':')
+             && ! is_pseudo_class_element(name_);
     }
+
+    // A pseudo-element is made of two colons (::) followed by the name.
+    // The `::` notation is introduced by the current document in order to
+    // establish a discrimination between pseudo-classes and pseudo-elements.
+    // For compatibility with existing style sheets, user agents must also
+    // accept the previous one-colon notation for pseudo-elements introduced
+    // in CSS levels 1 and 2 (namely, :first-line, :first-letter, :before and
+    // :after). This compatibility is not allowed for the new pseudo-elements
+    // introduced in this specification.
     virtual bool is_pseudo_element()
     {
-      if (name() == ":before"       || name() == "::before"     ||
-          name() == ":after"        || name() == "::after"      ||
-          name() == ":first-line"   || name() == "::first-line" ||
-          name() == ":first-letter" || name() == "::first-letter") {
-        return true;
-      }
-      else {
-        // If it's not a known pseudo-element, check whether it looks like one. This is similar to the type method on the Pseudo class in ruby sass.
-        return name().find("::") == 0;
-      }
+      return (name_[0] == ':' && name_[1] == ':')
+             || is_pseudo_class_element(name_);
+    }
+    virtual unsigned long specificity()
+    {
+      if (is_pseudo_element())
+        return Constants::Specificity_Type;
+      return Constants::Specificity_Pseudo;
     }
     virtual Compound_Selector* unify_with(Compound_Selector*, Context&);
     ATTACH_OPERATIONS();
@@ -1858,6 +1977,12 @@ namespace Sass {
     Wrapped_Selector(ParserState pstate, string n, Selector* sel)
     : Simple_Selector(pstate), name_(n), selector_(sel)
     { }
+    // Selectors inside the negation pseudo-class are counted like any
+    // other, but the negation itself does not count as a pseudo-class.
+    virtual unsigned long specificity()
+    {
+      return selector_ ? selector_->specificity() : 0;
+    }
     ATTACH_OPERATIONS();
   };
 
@@ -1886,7 +2011,7 @@ namespace Sass {
     { }
 
     Compound_Selector* unify_with(Compound_Selector* rhs, Context& ctx);
-    virtual Selector_Placeholder* find_placeholder();
+    // virtual Selector_Placeholder* find_placeholder();
     Simple_Selector* base()
     {
       // Implement non-const in terms of const. Safe to const_cast since this method is non-const
@@ -1898,7 +2023,7 @@ namespace Sass {
       return 0;
     }
     bool is_superselector_of(Compound_Selector* rhs);
-    virtual int specificity()
+    virtual unsigned long specificity()
     {
       int sum = 0;
       for (size_t i = 0, L = length(); i < L; ++i)
@@ -1933,7 +2058,6 @@ namespace Sass {
   // CSS selector combinators (">", "+", "~", and whitespace). Essentially a
   // linked list.
   ////////////////////////////////////////////////////////////////////////////
-  struct Context;
   class Complex_Selector : public Selector {
   public:
     enum Combinator { ANCESTOR_OF, PARENT_OF, PRECEDES, ADJACENT_TO };
@@ -1943,9 +2067,9 @@ namespace Sass {
     ADD_PROPERTY(Complex_Selector*, tail);
   public:
     Complex_Selector(ParserState pstate,
-                         Combinator c,
-                         Compound_Selector* h,
-                         Complex_Selector* t)
+                     Combinator c,
+                     Compound_Selector* h,
+                     Complex_Selector* t)
     : Selector(pstate), combinator_(c), head_(h), tail_(t)
     {
       if ((h && h->has_reference())   || (t && t->has_reference()))   has_reference(true);
@@ -1957,10 +2081,10 @@ namespace Sass {
     size_t length();
     bool is_superselector_of(Compound_Selector*);
     bool is_superselector_of(Complex_Selector*);
-    virtual Selector_Placeholder* find_placeholder();
+    // virtual Selector_Placeholder* find_placeholder();
     Combinator clear_innermost();
     void set_innermost(Complex_Selector*, Combinator);
-    virtual int specificity() const
+    virtual unsigned long specificity() const
     {
       int sum = 0;
       if (head()) sum += head()->specificity();
@@ -2020,7 +2144,7 @@ namespace Sass {
     }
     Complex_Selector* clone(Context&) const;      // does not clone Compound_Selector*s
     Complex_Selector* cloneFully(Context&) const; // clones Compound_Selector*s
-    vector<Compound_Selector*> to_vector();
+    // vector<Compound_Selector*> to_vector();
     ATTACH_OPERATIONS();
   };
 
@@ -2029,21 +2153,21 @@ namespace Sass {
   ///////////////////////////////////
   // Comma-separated selector groups.
   ///////////////////////////////////
-  class Selector_List
-      : public Selector, public Vectorized<Complex_Selector*> {
+  class Selector_List : public Selector, public Vectorized<Complex_Selector*> {
 #ifdef DEBUG
     ADD_PROPERTY(string, mCachedSelector);
 #endif
+    ADD_PROPERTY(vector<string>, wspace);
   protected:
     void adjust_after_pushing(Complex_Selector* c);
   public:
     Selector_List(ParserState pstate, size_t s = 0)
-    : Selector(pstate), Vectorized<Complex_Selector*>(s)
+    : Selector(pstate), Vectorized<Complex_Selector*>(s), wspace_(0)
     { }
-    virtual Selector_Placeholder* find_placeholder();
-    virtual int specificity()
+    // virtual Selector_Placeholder* find_placeholder();
+    virtual unsigned long specificity()
     {
-      int sum = 0;
+      unsigned long sum = 0;
       for (size_t i = 0, L = length(); i < L; ++i)
       { sum += (*this)[i]->specificity(); }
       return sum;
