@@ -27,6 +27,7 @@ namespace Sass {
     new_env.link(*env);
     env = &new_env;
     Block* bb = new (ctx.mem) Block(b->pstate(), b->length(), b->is_root());
+    // bb->tabs(b->tabs());
     block_stack.push_back(bb);
     append_block(b);
     block_stack.pop_back();
@@ -86,7 +87,7 @@ namespace Sass {
 
     Keyframe_Rule* rr = new (ctx.mem) Keyframe_Rule(r->pstate(),
                                                     r->block()->perform(this)->block());
-    if (r->rules()) rr->rules(r->rules());
+    if (r->selector()) rr->selector(r->selector());
 
     return debubble(rr->block(), rr)->block();
   }
@@ -97,25 +98,21 @@ namespace Sass {
     Ruleset* rr = new (ctx.mem) Ruleset(r->pstate(),
                                         r->selector(),
                                         r->block()->perform(this)->block());
+    // rr->tabs(r->block()->tabs());
     p_stack.pop_back();
 
-    Block* props = new Block(rr->block()->pstate());
-    for (size_t i = 0, L = rr->block()->length(); i < L; i++)
-    {
-      Statement* s = (*rr->block())[i];
-      if (!bubblable(s)) *props << s;
-    }
-
-    Block* rules = new Block(rr->block()->pstate());
+    Block* props = new (ctx.mem) Block(rr->block()->pstate());
+    Block* rules = new (ctx.mem) Block(rr->block()->pstate());
     for (size_t i = 0, L = rr->block()->length(); i < L; i++)
     {
       Statement* s = (*rr->block())[i];
       if (bubblable(s)) *rules << s;
+      if (!bubblable(s)) *props << s;
     }
 
     if (props->length())
     {
-      Block* bb = new Block(rr->block()->pstate());
+      Block* bb = new (ctx.mem) Block(rr->block()->pstate());
       *bb += props;
       rr->block(bb);
 
@@ -191,6 +188,7 @@ namespace Sass {
     {
       Block* bb = m->block()->perform(this)->block();
       for (size_t i = 0, L = bb->length(); i < L; ++i) {
+        // (bb->elements())[i]->tabs(m->tabs());
         if (bubblable((*bb)[i])) (*bb)[i]->tabs((*bb)[i]->tabs() + m->tabs());
       }
       if (bb->length() && bubblable(bb->last())) bb->last()->group_end(m->group_end());
@@ -208,7 +206,7 @@ namespace Sass {
   Statement* Cssize::bubble(At_Rule* m)
   {
     Block* bb = new (ctx.mem) Block(this->parent()->pstate());
-    Has_Block* new_rule = static_cast<Has_Block*>(this->parent());
+    Has_Block* new_rule = static_cast<Has_Block*>(shallow_copy(this->parent()));
     new_rule->block(bb);
     new_rule->tabs(this->parent()->tabs());
 
@@ -232,7 +230,7 @@ namespace Sass {
   Statement* Cssize::bubble(At_Root_Block* m)
   {
     Block* bb = new (ctx.mem) Block(this->parent()->pstate());
-    Has_Block* new_rule = static_cast<Has_Block*>(this->parent());
+    Has_Block* new_rule = static_cast<Has_Block*>(shallow_copy(this->parent()));
     new_rule->block(bb);
     new_rule->tabs(this->parent()->tabs());
 
@@ -252,7 +250,7 @@ namespace Sass {
 
   Statement* Cssize::bubble(Feature_Block* m)
   {
-    Ruleset* parent = static_cast<Ruleset*>(this->parent());
+    Ruleset* parent = static_cast<Ruleset*>(shallow_copy(this->parent()));
 
     Block* bb = new (ctx.mem) Block(parent->block()->pstate());
     Ruleset* new_rule = new (ctx.mem) Ruleset(parent->pstate(),
@@ -270,13 +268,15 @@ namespace Sass {
                                                     m->feature_queries(),
                                                     wrapper_block);
 
+    mm->tabs(m->tabs());
+
     Bubble* bubble = new (ctx.mem) Bubble(mm->pstate(), mm);
     return bubble;
   }
 
   Statement* Cssize::bubble(Media_Block* m)
   {
-    Ruleset* parent = static_cast<Ruleset*>(this->parent());
+    Ruleset* parent = static_cast<Ruleset*>(shallow_copy(this->parent()));
 
     Block* bb = new (ctx.mem) Block(parent->block()->pstate());
     Ruleset* new_rule = new (ctx.mem) Ruleset(parent->pstate(),
@@ -295,6 +295,8 @@ namespace Sass {
                                                 wrapper_block,
                                                 m->selector());
 
+    mm->tabs(m->tabs());
+
     Bubble* bubble = new (ctx.mem) Bubble(mm->pstate(), mm);
 
     return bubble;
@@ -496,7 +498,7 @@ namespace Sass {
 
   Media_Query* Cssize::merge_media_query(Media_Query* mq1, Media_Query* mq2)
   {
-    To_String to_string;
+    To_String to_string(&ctx);
 
     string type;
     string mod;
