@@ -5,39 +5,24 @@
 
 #include "position.hpp"
 #include "operation.hpp"
-#include "source_map.hpp"
+#include "emitter.hpp"
 
 namespace Sass {
+  class Context;
   using namespace std;
-  struct Context;
 
-  class Inspect : public Operation_CRTP<void, Inspect> {
+  class Inspect : public Operation_CRTP<void, Inspect>, public Emitter {
+  protected:
     // import all the class-specific methods and override as desired
     using Operation_CRTP<void, Inspect>::operator();
 
-    // To_String* to_string;
-    string buffer;
-    size_t indentation;
-    Context* ctx;
-    bool in_declaration;
-    bool in_declaration_list;
-
     void fallback_impl(AST_Node* n);
 
   public:
-    void append_indent_to_buffer();
-    void append_to_buffer(const string& text);
-    void append_to_buffer(const string& text, AST_Node* node);
-    void append_to_buffer(const string& text, AST_Node* node, const string& tail);
 
-  public:
-
-    SourceMap source_map;
-    Inspect(Context* ctx = 0);
+    Inspect(Emitter emi);
     virtual ~Inspect();
 
-    string get_buffer() { return buffer; }
-
     // statements
     virtual void operator()(Block*);
     virtual void operator()(Ruleset*);
@@ -79,12 +64,14 @@ namespace Sass {
     virtual void operator()(Boolean*);
     virtual void operator()(String_Schema*);
     virtual void operator()(String_Constant*);
+    virtual void operator()(String_Quoted*);
     virtual void operator()(Feature_Query*);
     virtual void operator()(Feature_Query_Condition*);
     virtual void operator()(Media_Query*);
     virtual void operator()(Media_Query_Expression*);
     virtual void operator()(At_Root_Expression*);
     virtual void operator()(Null*);
+    virtual void operator()(Parent_Selector* p);
     // parameters and arguments
     virtual void operator()(Parameter*);
     virtual void operator()(Parameters*);
@@ -103,12 +90,9 @@ namespace Sass {
     virtual void operator()(Complex_Selector*);
     virtual void operator()(Selector_List*);
 
-    template <typename U>
-    void fallback(U x) { fallback_impl(reinterpret_cast<AST_Node*>(x)); }
+    // template <typename U>
+    // void fallback(U x) { fallback_impl(reinterpret_cast<AST_Node*>(x)); }
   };
 
-  string unquote(const string&);
-  string quote(const string&, char);
-
 }
 #endif
