@@ -4,6 +4,15 @@
 #include <stddef.h>
 #include <stdbool.h>
 
+#ifdef __GNUC__
+  #define DEPRECATED(func) func __attribute__ ((deprecated))
+#elif defined(_MSC_VER)
+  #define DEPRECATED(func) __declspec(deprecated) func
+#else
+  #pragma message("WARNING: You need to implement DEPRECATED for this compiler")
+  #define DEPRECATED(func) func
+#endif
+
 #ifdef _WIN32
 
   /* You should define ADD_EXPORTS *only* when building the DLL. */
@@ -23,11 +32,8 @@
 
 #endif
 
-#ifndef LIBSASS_VERSION
-#define LIBSASS_VERSION "[NA]"
-#endif
-
 // include API headers
+#include "sass_version.h"
 #include "sass_values.h"
 #include "sass_functions.h"
 
@@ -46,13 +52,15 @@ enum Sass_Output_Style {
 };
 
 // Some convenient string helper function
-ADDAPI char* ADDCALL sass_string_quote (const char *str, const char quotemark);
-ADDAPI char* ADDCALL sass_string_unquote (const char *str);
+ADDAPI char* ADDCALL sass_string_quote (const char* str, const char quote_mark);
+ADDAPI char* ADDCALL sass_string_unquote (const char* str);
+
+// Resolve a file via the given include paths in the include char* array
+ADDAPI char* ADDCALL sass_resolve_file (const char* path, const char* incs[]);
 
 // Get compiled libsass version
 ADDAPI const char* ADDCALL libsass_version(void);
 
-
 #ifdef __cplusplus
 } // __cplusplus defined.
 #endif
